﻿using System;
using System.Collections;
using System.Linq;
using System.Text;

namespace SCCTestRuns
{
    class Program
    {
        
        static void Main(string[] args)
        {
           
            //Surround SCM vars - ideally these should come from a config file

            string scmUser = "Administrator";
            string scmPassword = "password";
            string scmServer = "127.0.0.1:4900";

            string scmRepo = "WysiCorp";
            string scmBranch = "WysiCorp";

            //TestTrack vars - we'll use the same user and password as for Surround SCM

            string ttURL = "http://localhost/scripts/ttsoapcgi.exe";
            string ttProjectName = "Sample Project";
            

            string ttTestRunSet = "Nightly Tests";


            long ttCookie = 0;
            //Process dates
            TimeSpan timesp = new TimeSpan(24,0,0);//Format of timespan is (h,m,s)
            
            DateTime curDate = DateTime.Now;
            DateTime modDate = DateTime.Now.Subtract(timesp);

            string pcurDate = ProcessDate(curDate);
            string pmodDate = ProcessDate(modDate);
            Console.WriteLine("Checking Surround SCM for changed files...");
            
            string scmCommand = "cc -b" + "\"" + scmBranch + "\"" + " -p" + "\"" + scmRepo + "\"" + " -d" + pmodDate + ":" + pcurDate + " -r" + " -y" + scmUser + ":" + scmPassword + " -z" + scmServer;

            string scmCmdResult = sscmexe.sscmCMD(scmCommand);
            //if the output of the command is a string less than 10 characters in lenght, then it probably did not detect any changes.
            if (scmCmdResult.Length < 10)
            {
                Console.WriteLine("No Changes detected");
            }
            else
            {
                Console.WriteLine("Changes detected...");
                //We'll start defining the two arrays we'll use
                //Array that will hold the files
                ArrayList fileList = new ArrayList();
                //Array that will hold the test case numbers that have links to the files
                ArrayList tCaseList = new ArrayList();
                Console.WriteLine("Building file list...");
                //The command output and branch name are sent to the ProcessSCMOutput() method, which returns the Array filled for us
                fileList = ProcessSCMOutput(scmCmdResult, scmBranch);
                Console.WriteLine("Build list of files.  List contains a total of " + fileList.Count.ToString() + " files");
                Console.WriteLine("Checking with TestTrack for possible matches...");
                //Now that we have our list of files in a format that TestTrack will understand we can query the test cases.
                //Start the soap stuff
                ttsoapcgi ttSoap = new ttsoapcgi();
                //set the url
                ttSoap.Url = ttURL;
                //Locate the project
                CProject ttCProject = null;
                try
                {
                    CProject[] ttCProjects = ttSoap.getProjectList(scmUser, scmPassword);
                    for (int i = 0; i < ttCProjects.Length; i++)
                    {
                        if (ttCProjects[i].database.name.Equals(ttProjectName))
                        {
                            ttCProject = ttCProjects[i];

                        }
                    }
                }
                catch (Exception fcc)
                {
                    Console.WriteLine("Unable to get list of projects for user.");
                    Console.WriteLine("The error returned is: " + fcc.Message);
                    
                   
                }
                if (ttCProject != null)
                {
                    try
                    {
                        //if we found project for user, lets log in
                        ttCookie = ttSoap.ProjectLogon(ttCProject, scmUser, scmPassword);
                        Console.WriteLine("Logged in to project...");
                        CTableColumn[] ttColumns = new CTableColumn[1];
                        ttColumns[0] = new CTableColumn();
                        ttColumns[0].name = "Number";
                        CRecordListSoap ttRecs = ttSoap.getRecordListForTable(ttCookie,"Test Case","",ttColumns);
                        //If we have then's let's proceed!!!
                        if ((!(ttRecs == null)) && (ttRecs.records.Length > 0))
                        {
                            Console.WriteLine("Got list of test cases...");
                            //for each test case number, check for files
                            for (int i = 0; i < ttRecs.records.Length; i++)
                            {
                                Console.WriteLine("Checking test case " + ttRecs.records[i].row[0].value.ToString());
                                bool contains = false;
                                CTestCase tCase = ttSoap.getTestCase(ttCookie,Convert.ToInt64(ttRecs.records[i].row[0].value), "", false);
                                if (tCase.sccFileList != null)
                                {
                                    for (int j = 0; j < tCase.sccFileList.Length; j++)
                                    {
                                        for (int k = 0; k < fileList.Count; k++)
                                        {
                                            if (tCase.sccFileList[j].mstrFileName == fileList[k].ToString())
                                            {
                                                contains = true;
                                               
                                            }
                                        }
                                    }
                                    if (contains)
                                    {
                                        //if test case contains changed scc files, add test case number to array.
                                        tCaseList.Add(tCase.testcasenumber);
                                    }
                                }
                            }
                            if (tCaseList.Count > 0)
                            {
                                //if array has 1 or more elements, then we did find some matches.
                                Console.WriteLine("Found a total of " + tCaseList.Count.ToString() + " match(es)");
                                for (int i = 0; i < tCaseList.Count; i++)
                                {
                                    CTestCase ttTCase = ttSoap.getTestCase(ttCookie, Convert.ToInt64(tCaseList[i]), "", false);
                                    //before we can generate the test run for the test case, we need
                                    //to build our test variants.  We'll use all variants defined in the test case
                                    CTestRunVariantField[] ttRunVariants = new CTestRunVariantField[ttTCase.testVariants.Length];
                                    for (int k = 0; k < ttTCase.testVariants.Length; k++)
                                    {
                                        //Need to make sure we only look at the "Included" variants, and not the "Excluded" ones.
                                        if (ttTCase.testVariants[k].type == "Included")
                                        {
                                            CFieldValue[] tvVal = new CFieldValue[ttTCase.testVariants[k].values.Length];
                                            //Go through each value in this variant 
                                            for (int l = 0; l < ttTCase.testVariants[k].values.Length; l++)
                                            {
                                                tvVal[l] = new CFieldValue();
                                                tvVal[l].value = ttTCase.testVariants[k].values[l].value;
                                            }

                                            ttRunVariants[k] = new CTestRunVariantField();
                                            ttRunVariants[k].name = ttTCase.testVariants[k].name;
                                            ttRunVariants[k].values = tvVal;
                                        }
                                    }
                                    //Now we can generate the test run 
                                    try
                                    {

                                        CItemToTrack[] ttRuns = ttSoap.generateTestRuns(ttCookie, ttTCase.recordid, ttRunVariants, ttTestRunSet);
                                        Console.WriteLine("Created test run(s) for test case " + ttTCase.testcasenumber);
                                    }
                                    catch (Exception fcc)
                                    {
                                        Console.WriteLine("Unable to generate test run for test case " + ttTCase.testcasenumber.ToString() + ".");
                                        Console.WriteLine("The error returned is: " + fcc.Message);

                                    }

                                }
                            }
                            else
                            {
                                Console.WriteLine("Did not find any matches");
                            }
                        }
                        else
                        {
                            Console.WriteLine("The filter did not return any defects!");
                        }

                        

                        

                        ttSoap.DatabaseLogoff(ttCookie);
                    }
                    catch (Exception fcc)
                    {
                        Console.WriteLine("Error encountered when trying to proccess test runs.");
                        Console.WriteLine("The error returned is: " + fcc.Message);
                    }
                }
            }
            


        }

        //Converts date from .NET format to CC command format
        public static string ProcessDate(DateTime dateToProcess)
        {
            string procDate = "";

            procDate += dateToProcess.Year.ToString();
            if (dateToProcess.Month < 10)
            {
                procDate += "0" + dateToProcess.Month.ToString();
            }
            else
            {
                procDate += dateToProcess.Month.ToString();
            }
            if (dateToProcess.Day < 10)
            {
                procDate += "0" + dateToProcess.Day.ToString();
            }
            else
            {
                procDate += dateToProcess.Day.ToString();
            }

            if (dateToProcess.Hour < 10)
            {
                procDate += "0" + dateToProcess.Hour.ToString();
            }
            else
            {
                procDate += dateToProcess.Hour.ToString();
            }
            if (dateToProcess.Minute < 10)
            {
                procDate += "0" + dateToProcess.Minute.ToString();
            }
            else
            {
                procDate += dateToProcess.Minute.ToString();
            }
            if (dateToProcess.Second < 10)
            {
                procDate += "0" + dateToProcess.Second.ToString();
            }
            else
            {
                procDate += dateToProcess.Second.ToString();
            }


            return procDate;

        }

        //Builds the array of Surround SCM files.
        public static ArrayList ProcessSCMOutput(string outToProc, string scmBranch)
        {
            ArrayList fileList = new ArrayList();
            bool flag = true;
            while (flag)
            {
                outToProc = outToProc.Substring(outToProc.IndexOf("<"));
                string tempString = outToProc.Substring(0, outToProc.IndexOf("\n") - 1);
                tempString = ProcessFile(tempString);
                fileList.Add(scmBranch + "::" + tempString);
                outToProc = outToProc.Substring(outToProc.IndexOf("\n") + 1);
                if (outToProc.Length < 2)
                {
                    flag = false;
                }
            }
            return fileList;
        }
        //Parses output from CC command into the format that TestTrack will understand
        public static string ProcessFile(string fileToProc)
        {
            string procFile = "";

            fileToProc = fileToProc.Substring(fileToProc.IndexOf("<") + 1);
            procFile = fileToProc.Substring(0, fileToProc.IndexOf(">") );
            fileToProc = fileToProc.Substring(fileToProc.IndexOf(">"));
            fileToProc = fileToProc.Substring(fileToProc.IndexOf("<"));
            procFile += "/" + fileToProc.Substring(1, fileToProc.IndexOf(">")-1 );

            return procFile;
        }
    }
}
